use crate::models::characters::{CharacterEntry, NewCharacter};
use crate::models::users::{NewUser, User};
use diesel::prelude::*;
use diesel::SqliteConnection;

#[database("tenebrous_db")]
pub(crate) struct TenebrousDbConn(SqliteConnection);

pub(crate) trait Dao {
    fn load_user_by_id(&self, id: i32) -> QueryResult<Option<User>>;
    fn load_user(&self, for_username: &str) -> QueryResult<Option<User>>;

    fn insert_user(&self, new_user: &NewUser) -> QueryResult<User>;

    fn load_character_list(&self, for_user_id: i32) -> QueryResult<Vec<CharacterEntry>>;

    fn load_character(&self, character_id: i32) -> QueryResult<Option<CharacterEntry>>;

    fn insert_character(&self, new_character: &NewCharacter) -> QueryResult<()>;
}

impl Dao for TenebrousDbConn {
    fn load_user_by_id(&self, user_id: i32) -> QueryResult<Option<User>> {
        use crate::schema::users::dsl::*;

        users.filter(id.eq(user_id)).first(&self.0).optional()
    }

    fn load_user(&self, for_username: &str) -> QueryResult<Option<User>> {
        use crate::schema::users::dsl::*;

        users
            .filter(username.eq(for_username))
            .first(&self.0)
            .optional()
    }

    fn insert_user(&self, new_user: &NewUser) -> QueryResult<User> {
        use crate::schema::users;

        diesel::insert_into(users::table)
            .values(new_user)
            .execute(&self.0)?;

        use crate::schema::users::dsl::*;
        Ok(users
            .filter(username.eq(new_user.username))
            .first(&self.0)?)
    }

    fn load_character_list(&self, for_user_id: i32) -> QueryResult<Vec<CharacterEntry>> {
        use crate::schema::characters::dsl::*;
        characters.filter(user_id.eq(for_user_id)).load(&self.0)
    }

    fn load_character(&self, character_id: i32) -> QueryResult<Option<CharacterEntry>> {
        use crate::schema::characters::dsl::*;

        characters
            .filter(id.eq(character_id))
            .first(&self.0)
            .optional()
    }

    fn insert_character(&self, new_character: &NewCharacter) -> QueryResult<()> {
        use crate::schema::characters;

        diesel::insert_into(characters::table)
            .values(new_character)
            .execute(&self.0)?;

        Ok(())
    }
}
